'use strict';
module.exports = {
  name: "wallet",
  title: "加盟商余额",
  vuex: true,
  model: [
    {
      title: "加盟商余额调整",
      mapFn: "correctPartnerAmount",
      path: "/finance-partner/v1/wallet/correct-partner-amount",
      isRule: false,
      data: {
        partner_id: ["加盟商ID", "string", "必填"],
        amount: ["调整金额", "单位分", "必填"],
        correct_type: ["调整类型", "correct_a:上调；correct_r:下调", "必填"],
        wallet_nnid: ["钱包唯一标识", "string", "必填"],
      },
      disabled: true,
      "role": "FINANCE-PARTNER.WALLET_CORRECT_PARTNER_AMOUNT"
    },
    {
      title: "平台(供货商)余额调整",
      mapFn: "correctSupplierAmount",
      path: "/finance-partner/v1/wallet/correct-supplier-amount",
      isRule: false,
      data: {
        supplier_id: ["供货商ID", "string", "必填"],
        amount: ["调整金额", "单位分", "必填"],
        correct_type: ["调整类型", "correct_a:上调；correct_r:下调", "必填"],
        wallet_nnid: ["钱包唯一标识", "string", "必填"],
      },
      disabled: true,
      "role": "FINANCE-PARTNER.WALLET_CORRECT_SUPPLIER_AMOUNT"
    },
    {
      title: "获取渠道余额",
      mapFn: "getChannelAmount",
      path: "/finance-partner/v1/wallet/get-channel-amount",
      isRule: false,
      params: {
        supplier_id: ["供货商ID", "string", "必填"],
        channel_code: ["渠道编码", "string", "必填"],
      },
      disabled: true,
      "role": "FINANCE-PARTNER.WALLET_GET_CHANNEL_AMOUNT"
    },
    {
      title: "渠道账户余额列表",
      mapFn: "listChannelWallet",
      path: "/finance-partner/v1/wallet/list-channel-wallet",
      isRule: false,
      params: {
        supplier_id: ["供货商ID", "string", "必填"],
        supplier_name: ["供货商名称", "string", "非必填"],
        page_size: ["每页条数", "number", "非必填"],
        page_token: ["翻页标识", "string", "非必填"],
      },
      disabled: true,
      "role": "FINANCE-PARTNER.WALLET_LIST_CHANNEL_WALLET"
    },
    {
      title: "加盟商账户余额列表",
      mapFn: "listPartnerWallet",
      path: "/finance-partner/v1/wallet/list-partner-wallet",
      isRule: false,
      params: {
        partner_id: ["加盟商ID", "string", "必填"],
        partner_name: ["加盟商名称", "string", "非必填"],
        wallet_type: ["钱包类型", "string", "非必填"],
        page_size: ["每页条数", "number", "非必填"],
        page_token: ["翻页标识", "string", "非必填"],
      },
      disabled: true,
      "role": "FINANCE-PARTNER.WALLET_LIST_PARTNER_WALLET"
    },
    {
      title: "加盟商首页余额列表",
      mapFn: "listHomeWallet",
      path: "/finance-partner/v1/wallet/list-home-wallet",
      isRule: false,
      params: {},
      disabled: true,
      "role": "FINANCE-PARTNER.WALLET_LIST_HOME_WALLET"
    },
    {
      title: "供货商账户余额列表",
      mapFn: "listSupplierWallet",
      path: "/finance-partner/v1/wallet/list-supplier-wallet",
      isRule: false,
      params: {
        supplier_id: ["供货商ID", "string", "必填"],
        supplier_name: ["供货商名称", "string", "非必填"],
        settlement_method: ["结算方式", "string", "非必填"],
        page_size: ["每页条数", "number", "非必填"],
        page_token: ["翻页标识", "string", "非必填"],
      },
      disabled: true,
      "role": "FINANCE-PARTNER.WALLET_LIST_SUPPLIER_WALLET"
    },
    {
      title: "open API余额列表",
      mapFn: "listOpenApiWallet",
      path: "/finance-partner/v1/wallet/list-openapi-wallet",
      isRule: false,
      params: {
        open_user_id: ["渠道用户ID", "string", "必填"],
        open_user_name: ["用户名", "string", "非必填"],
        settlement_method: ["结算方式", "string", "非必填"],
        page_size: ["每页条数", "number", "非必填"],
        page_token: ["翻页标识", "string", "非必填"],
      },
      disabled: true,
      "role": "FINANCE-PARTNER.WALLET_LIST_OPENAPI_WALLET"
    }
  ]
}